@@ -201,8 +201,7 @@ def ToList(input):
 
 _USED_DIRS = set()
 
-def _PyScript(env, target, source, command = "python $SOURCES", 
-                    capture=None, execute=None, tempdir=None, lightweight=True, *args, **kwargs): 
+def _PyScript(env, target, source, command = "python $SOURCES", capture=None, execute=None, tempdir=None, lightweight=True, *args, **kwargs): 
     """
     A new Builder added to a scons environment which has the same syntax as Command.
     """
@@ -219,17 +218,17 @@ def _PyScript(env, target, source, command = "python $SOURCES",
                                     % module).read().strip()
         if s[-3:] in ["pyc", "pyo"]: s = s[:-1]
         source[n] = s
-    
+
     if capture in ["stdout", "out", "output"]: command[-1] += " > $TARGET"
     elif capture in ["stderr", "err", "error"]: command[-1] += " 2> $TARGET"
     elif capture == "join": command[-1] += " &> $TARGET"
     elif capture == "both": command[-1] += " > ${TARGETS[0]} 2> ${TARGETS[1]}"
     elif capture == None: pass
-   
+
     command_sig = [ env.subst(c, target=target, source=subst) for c in command ] 
 
     basename = None
-    if execute and tempdir == None: 
+    if execute and tempdir == None:
         m = hashlib.sha1()
         m.update(tuple(command_sig))
         basename = os.path.join("temp", base64.b64encode(m.digest(), "+="), '')
@@ -241,9 +240,7 @@ def _PyScript(env, target, source, command = "python $SOURCES",
         if not os.path.exists(basename): mkdir_p(basename)
         scons = os.path.join(basename, "SConscript")
         file = open(scons, 'w')
-        print >>file, """from pyscons import * 
-targets, sources, basename = PySconsSetup()
-PyScript(targets, sources, %s) """ % repr(command)
+        print >>file, "from pyscons import *\ntargets, sources, basename = PySconsSetup()\nPyScript(targets, sources, %s)" % repr(command)
         file.close()
 
         arg = " ".join(["in%g=%s" % (n+1, str(s)) for n,s in enumerate(source)])
@@ -327,31 +324,31 @@ def PySconsSetup(env=None, args=ARGUMENTS, tools=['default', PYTOOL()]):
 
     sources = []
     targets = []
+
     for a in args:
         if a[:2] == "in":
-            try:
-                i = int(a[2:])
-                sources.append((i, args[a]))
+            try: sources.append((int(a[2:]), args[a]))
             except: pass
         if a[:3] == "out":
-            try:
-                i = int(a[3:])
-                targets.append((i, args[a]))
+            try: targets.append((int(a[3:]), args[a]))
             except: pass
 
     myname = args["n"]
     N = len(os.path.split(myname))
 
+    if "exclude" in args: #exlude files names listed in this file from SHIP.tz2
+        exclude = set(L.strip() for L in file(args["exclude"]))
+    else: exclude = set()
+
+    basename = args["temp"]
+    bn = list(os.path.split(basename))
+    env.SConsignFile(os.path.join(*([".."] * N + bn + [".sconsign"])))
+
     sources = [a for i,a in sorted(sources)]
     targets = [a for i,a in sorted(targets)]
 
     env.NoClean(targets)
-    
-    try: basename = args["temp"] 
-    except: basename = None
-
-    bn = list(os.path.split(basename))
-    env.SConsignFile(os.path.join(*([".."] * N + bn + [".sconsign"])))
+    env.Default(targets)
 
     ftargets = [t + ".force" for t in targets]
     if reduce(lambda a,b: a&b, (os.path.exists(t) for t in ftargets)):
@@ -364,6 +361,8 @@ def PySconsSetup(env=None, args=ARGUMENTS, tools=['default', PYTOOL()]):
     out = os.path.join(basename, "OUT.tz2")
     force = os.path.join(basename, "IN.tz2")
     exe = os.path.join(basename, "EXECUTE")
+    SIGNATURES = os.path.join(basename, ".sconsign.dblite")
+    ship_source = [s for s in sources if s not in exclude] + [exe, myname]
 
     if os.path.exists(force):
         import subprocess
@@ -373,10 +372,19 @@ def PySconsSetup(env=None, args=ARGUMENTS, tools=['default', PYTOOL()]):
         shutil.mv(force, out)
         env.Exit(0)
 
-    env.Command(ship, sources + [exe, myname], "tar cjf $TARGET $SOURCES")
-    env.Default(targets)
-    SIGNATURES = os.path.join(basename, ".sconsign.dblite")
+    env.Command(ship, ship_source, "tar cjf $TARGET $SOURCES")
     env.Command(out, targets, "tar cjf $TARGET $SOURCES")
 
-    return targets, sources, basename
+    if args.get("print", None) == "sources":
+        for s in sources: print s
+        env.Exit(0)
 
+    if args.get("print", None) == "targets":
+        for t in targets: print t
+        env.Exit(0)
+
+    if args.get("print", None) == "ship":
+        for s in ship_source: print s
+        env.Exit(0)
+
+    return targets, sources, basename
